@@ -1,19 +1,22 @@
 import logging
 import sys
 import threading
+import typing
 from uuid import uuid4
 
-from hazelcast import six
+from hazelcast.compact import CompactSchemaService
 from hazelcast.errors import HazelcastError, HazelcastClientNotActiveError, TargetDisconnectedError
 from hazelcast.future import combine_futures, ImmediateFuture
 from hazelcast.invocation import Invocation
+from hazelcast.protocol.client_message import InboundMessage
 from hazelcast.protocol.codec import client_add_cluster_view_listener_codec
+from hazelcast.serialization.compact import SchemaNotFoundError
 from hazelcast.util import check_not_none
 
 _logger = logging.getLogger(__name__)
 
 
-class _ListenerRegistration(object):
+class _ListenerRegistration:
     __slots__ = (
         "registration_request",
         "decode_register_response",
@@ -32,7 +35,7 @@ class _ListenerRegistration(object):
         self.connection_registrations = {}  # Dict of Connection, EventRegistration
 
 
-class _EventRegistration(object):
+class _EventRegistration:
     __slots__ = ("server_registration_id", "correlation_id")
 
     def __init__(self, server_registration_id, correlation_id):
@@ -40,15 +43,23 @@ class _EventRegistration(object):
         self.correlation_id = correlation_id
 
 
-class ListenerService(object):
-    def __init__(self, client, config, connection_manager, invocation_service):
+class ListenerService:
+    def __init__(
+        self,
+        client,
+        config,
+        connection_manager,
+        invocation_service,
+        compact_schema_service: CompactSchemaService,
+    ):
         self._client = client
         self._connection_manager = connection_manager
         self._invocation_service = invocation_service
+        self._compact_schema_service = compact_schema_service
         self._is_smart = config.smart_routing
-        self._active_registrations = {}  # Dict of user_registration_id, ListenerRegistration
+        self._active_registrations: typing.Dict[str, _ListenerRegistration] = {}
         self._registration_lock = threading.RLock()
-        self._event_handlers = {}
+        self._event_handlers: typing.Dict[int, typing.Callable] = {}
 
     def start(self):
         self._connection_manager.add_listener(self._connection_added, self._connection_removed)
@@ -64,7 +75,7 @@ class ListenerService(object):
             self._active_registrations[registration_id] = registration
 
             futures = []
-            for connection in list(six.itervalues(self._connection_manager.active_connections)):
+            for connection in list(self._connection_manager.active_connections.values()):
                 future = self._register_on_connection(registration_id, registration, connection)
                 futures.append(future)
 
@@ -87,9 +98,10 @@ class ListenerService(object):
                 return ImmediateFuture(False)
 
             futures = []
-            for connection, event_registration in six.iteritems(
-                listener_registration.connection_registrations
-            ):
+            for (
+                connection,
+                event_registration,
+            ) in listener_registration.connection_registrations.items():
                 # Remove local handler
                 self.remove_event_handler(event_registration.correlation_id)
                 # The rest is for deleting the remote registration
@@ -129,13 +141,43 @@ class ListenerService(object):
             listener_registration.connection_registrations.clear()
             return combine_futures(futures).continue_with(lambda _: True)
 
-    def handle_client_message(self, message, correlation_id):
+    def handle_client_message(self, message: InboundMessage, correlation_id: int):
         handler = self._event_handlers.get(correlation_id, None)
         if handler:
-            handler(message)
+            try:
+                handler(message)
+            except SchemaNotFoundError as e:
+                self._fetch_schema_and_handle_again(e, handler, message)
         else:
             _logger.debug("Got event message with unknown correlation id: %s", message)
 
+    def _fetch_schema_and_handle_again(
+        self,
+        error: SchemaNotFoundError,
+        handler: typing.Callable[[InboundMessage], None],
+        message: InboundMessage,
+    ) -> None:
+        schema_id = error.schema_id
+
+        def callback(future):
+            try:
+                schema = future.result()
+                self._compact_schema_service.register_fetched_schema(schema_id, schema)
+            except:
+                _logger.exception(
+                    f"Failed to call event handler: {handler} with message: {message}"
+                )
+                return
+
+            message.reset_next_frame()
+            try:
+                handler(message)
+            except SchemaNotFoundError as e:
+                self._fetch_schema_and_handle_again(e, handler, message)
+
+        fetch_schema_future = self._compact_schema_service.fetch_schema(schema_id)
+        fetch_schema_future.add_done_callback(callback)
+
     def add_event_handler(self, correlation_id, event_handler):
         self._event_handlers[correlation_id] = event_handler
 
@@ -178,12 +220,12 @@ class ListenerService(object):
 
     def _connection_added(self, connection):
         with self._registration_lock:
-            for user_reg_id, listener_registration in six.iteritems(self._active_registrations):
+            for user_reg_id, listener_registration in self._active_registrations.items():
                 self._register_on_connection(user_reg_id, listener_registration, connection)
 
     def _connection_removed(self, connection):
         with self._registration_lock:
-            for listener_registration in six.itervalues(self._active_registrations):
+            for listener_registration in self._active_registrations.values():
                 event_registration = listener_registration.connection_registrations.pop(
                     connection, None
                 )
@@ -191,7 +233,7 @@ class ListenerService(object):
                     self.remove_event_handler(event_registration.correlation_id)
 
 
-class ClusterViewListenerService(object):
+class ClusterViewListenerService:
     def __init__(
         self,
         client,
